/*
PC7266 diagnostic software
This program makes sure the counters can be accessed on the PC7266 board.
8/20/96
*/

#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>
#include <math.h>


typedef unsigned char BYTE;
typedef unsigned int WORD;

#define DEF_BASE	0X300	/* default base address of interface board */

/* LS7266 addresses */
#define DATA1			0			/* data register of LS7266 */
#define CONTROL1	1			/* control register of LS7266 */
#define DATA2			2			/* data register of LS7266 */
#define CONTROL2	3			/* control register of LS7266 */
#define DATA3			4			/* data register of LS7266 */
#define CONTROL3	5			/* control register of LS7266 */
#define DATA4			6			/* data register of LS7266 */
#define CONTROL4	7			/* control register of LS7266 */
#define IRQ1			8 		/* IRQ control buffer 1 */
#define IRQ2			9 		/* IRQ control buffer 2 */
#define IOPORT    A			/* input/output port */

/* LS7266 commands */
#define CLOCK_DATA 		14				// FCK frequency divider
#define CLOCK_SETUP 	0X98			// transfer PR0 to PSC (x and y)
#define INPUT_SETUP 	0XC1			// enable inputs A and B (x and y)
#define QUAD_X1     	0XA8			// quadrature multiplier to 1 (x and y)
#define QUAD_X2     	0XB0			// quadrature multiplier to 2 (x and y)
#define QUAD_X4     	0XB8			// quadrature multiplier to 4 (x and y)
#define BP_RESET    	0X01			// reset byte pointer
#define BP_RESETB  		0X81			// reset byte pointer (x and y)
#define CNTR_RESET    0X02			// reset counter
#define CNTR_RESETB   0X82			// reset counter (x and y)
#define TRSFRPR_CNTR  0X08			// transfer preset register to counter
#define TRSFRCNTR_OL	0X90			// transfer CNTR to OL (x and y)
#define EFLAG_RESET		0X86			// reset E bit of flag register

/* Function prototypes */

int  test(void);			 /* test if counters can be accessed */

/* Global variables */

int  base = DEF_BASE;	 /* base address of interface board */

/* Main program */
int main(void)
{
	clrscr();

	printf("US Digital Corp.  PC7266 diagnostics");

	gotoxy(1,3); printf("Enter address of PC7266 card (hex): ");
	scanf("%x", &base);

	if (test() == 0) {
		printf("\n\nTest passed: the four counters can be accessed.");
	}

	return 0;
}



int test(void) {			 /* test if the counters can be accessed */

	int fail1 = 0, fail2 = 0, fail3 = 0, fail4 = 0;

	gotoxy(1,5);

	/* disable index */
	outportb(base + IRQ1 ,0);
	outportb(base + IRQ2 ,0);

	/* preset values to the preset register*/
	outportb(base + CONTROL1, BP_RESET);			// reset byte pointer
	outportb(base + DATA1, 0x13);							// output preset value
	outportb(base + DATA1, 0x57);
	outportb(base + DATA1, 0x0E);

	/*read counter back to test it*/
	outportb(base + CONTROL1, TRSFRPR_CNTR);		// transfer preset to counter
	outportb(base + CONTROL1, TRSFRCNTR_OL);		// transfer CNTR to OL
	outportb(base + CONTROL1, CNTR_RESET);			// reset counter
	outportb(base + CONTROL1, BP_RESET);				// reset byte pointer
	if (inportb(base + DATA1) != 0x13) fail1++; // read output latch
	if (inportb(base + DATA1) != 0x57) fail1++;	// if data is different, 1 is added to fail
	if (inportb(base + DATA1) != 0x0E) fail1++;
	if (fail1) printf("Counter 1 failed\n");		// print message if failed

	/* preset values to the preset register*/
	outportb(base + CONTROL2, BP_RESET);				// reset byte pointer
	outportb(base + DATA2, 0x78);								// output preset value
	outportb(base + DATA2, 0xFF);
	outportb(base + DATA2, 0xCB);

	/*read counter back to test it*/
	outportb(base + CONTROL2, TRSFRPR_CNTR);		// transfer preset to counter
	outportb(base + CONTROL2, TRSFRCNTR_OL);		// transfer CNTR to OL
	outportb(base + CONTROL2, CNTR_RESETB);			// reset counter (x and y)
	outportb(base + CONTROL2, BP_RESET);				// reset byte pointer
	if (inportb(base + DATA2) != 0x78) fail2++;	// read output latch
	if (inportb(base + DATA2) != 0xFF) fail2++;	// if data is different, 1 is added to fail
	if (inportb(base + DATA2) != 0xCB) fail2++;
	if (fail2) printf("Counter 2 failed\n");		// print message if failed

	/* preset values to the preset register*/
	outportb(base + CONTROL3, BP_RESET);				// reset byte pointer
	outportb(base + DATA3, 0xF2);								// output preset value
	outportb(base + DATA3, 0xA4);
	outportb(base + DATA3, 0x16);

	/*read counter back to test it*/
	outportb(base + CONTROL3, TRSFRPR_CNTR);		// transfer preset to counter
	outportb(base + CONTROL3, TRSFRCNTR_OL);		// transfer CNTR to OL
	outportb(base + CONTROL3, CNTR_RESET);			// reset counter
	outportb(base + CONTROL3, BP_RESET);				// reset byte pointer
	if (inportb(base + DATA3) != 0xF2) fail3++; // read output latch
	if (inportb(base + DATA3) != 0xA4
	) fail3++;	// if data is different, 1 is added to fail
	if (inportb(base + DATA3) != 0x16) fail3++;
	if (fail3) printf("Counter 3 failed\n");		// print message if failed

	/* preset values to the preset register*/
	outportb(base + CONTROL4, BP_RESET);				// reset byte pointer
	outportb(base + DATA4, 0xD8);								// output preset value
	outportb(base + DATA4, 0x00);
	outportb(base + DATA4, 0x9F);

	/*read counter back to test it*/
	outportb(base + CONTROL4, TRSFRPR_CNTR);		// transfer preset to counter
	outportb(base + CONTROL4, TRSFRCNTR_OL);		// transfer CNTR to OL
	outportb(base + CONTROL4, CNTR_RESETB);			// reset counter (x and y)
	outportb(base + CONTROL4, BP_RESET);				// reset byte pointer
	if (inportb(base + DATA4) != 0xD8) fail4++;	// read output latch
	if (inportb(base + DATA4) != 0x00) fail4++;	// if data is different, 1 is added to fail
	if (inportb(base + DATA4) != 0x9F) fail4++;
	if (fail4) printf("Counter 4 failed\n");		// print message if failed

	if (fail1 && fail2 && fail3 && fail4) {
		printf("\n\nAll counters failed, check the address on the DIP switch");
		printf("\nand make sure that address does not interfere with any other");
		printf("\ncards in your system.");
	}
	else if (fail1 || fail2 || fail3 || fail4) {
		printf("\n\nSome counters failed, check the address on the DIP switch");
		printf("\nand make sure that address does not interfere with any other");
		printf("\ncards in your system.");
	}

	return fail1 + fail2 + fail3 + fail4;
}
