/*
PC7266 demo software
This program interfaces to 4 encoders with the PC7266 board.
The position of each encoder is displayed in graphics mode.
This software requires a VGA adapter and the EGAVGA.BGI file
8/20/96
*/
char  version[] = "1.00";

#include <GRAPHICS.H>
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>
#include <time.h>
#include <math.h>


typedef unsigned char BYTE;
typedef unsigned int WORD;

#define DEF_BASE	0X300	/* default base address of interface board */

/* LS7266 addresses */
#define DATA1			0			/* data register of LS7266 */
#define CONTROL1	1			/* control register of LS7266 */
#define DATA2			2			/* data register of LS7266 */
#define CONTROL2	3			/* control register of LS7266 */
#define DATA3			4			/* data register of LS7266 */
#define CONTROL3	5			/* control register of LS7266 */
#define DATA4			6			/* data register of LS7266 */
#define CONTROL4	7			/* control register of LS7266 */
#define IRQ1			8 		/* IRQ control buffer 1 */
#define IRQ2			9 		/* IRQ control buffer 2 */
#define IOPORT    A			/* input/output port */

/* LS7266 commands */
#define CLOCK_DATA 		14				// FCK frequency divider
#define CLOCK_SETUP 	0X98			// transfer PR0 to PSC (x and y)
#define INPUT_SETUP 	0XC1			// enable inputs A and B (x and y)
#define QUAD_X1     	0XA8			// quadrature multiplier to 1 (x and y)
#define QUAD_X2     	0XB0			// quadrature multiplier to 2 (x and y)
#define QUAD_X4     	0XB8			// quadrature multiplier to 4 (x and y)
#define BP_RESET    	0X01			// reset byte pointer
#define BP_RESETB  		0X81			// reset byte pointer (x and y)
#define CNTR_RESET    0X02			// reset counter
#define CNTR_RESETB   0X82			// reset counter (x and y)
#define TRSFRPR_CNTR  0X08			// transfer preset register to counter
#define TRSFRCNTR_OL	0X90			// transfer CNTR to OL (x and y)
#define EFLAG_RESET		0X86			// reset E bit of flag register

/* Function prototypes	*/

void update_vectors(void);	/* update vectors in graphics mode */
void read_encoders(void);		/* read position of encoders */
void reset_encoders(void);	/* reset the counters */
int  test(void);						/* test if counters can be accessed */
int  init(void);						/* initializations */

/* Global variables */

int  base = DEF_BASE;		/* base address of interface board */
long encoder1_pos, encoder2_pos, encoder3_pos, encoder4_pos;
long encoder1_oldpos, encoder2_oldpos, encoder3_oldpos, encoder4_oldpos;
long encoder1_timepos, encoder2_timepos, encoder3_timepos, encoder4_timepos;
float encoder1_speed, encoder2_speed, encoder3_speed, encoder4_speed;
clock_t this_time, last_time;
long sign1 = 1, sign2 = 1, sign3 = 1, sign4 = 1;
int  multiplier = 4;
long resolution = 1;
int  graphics_mode = 0;


/* Main program */
int main(void)
{
	int ok = 1;

	if (init()) return 1;

	while (ok) {

		/* update encoder positions */
		read_encoders();
		gotoxy(2, 11); printf("%11ld", encoder1_pos);
		gotoxy(2, 15); printf("%11ld", encoder2_pos);
		gotoxy(2, 19); printf("%11ld", encoder3_pos);
		gotoxy(2, 23); printf("%11ld", encoder4_pos);
		gotoxy(2, 12); printf("%7.2f deg", (float)(encoder1_pos % (resolution*multiplier)) * 360.0 / (float)(resolution*multiplier));
		gotoxy(2, 16); printf("%7.2f deg", (float)(encoder2_pos % (resolution*multiplier)) * 360.0 / (float)(resolution*multiplier));
		gotoxy(2, 20); printf("%7.2f deg", (float)(encoder3_pos % (resolution*multiplier)) * 360.0 / (float)(resolution*multiplier));
		gotoxy(2, 24); printf("%7.2f deg", (float)(encoder4_pos % (resolution*multiplier)) * 360.0 / (float)(resolution*multiplier));
		gotoxy(2, 13); printf("%7.1f RPM", encoder1_speed);
		gotoxy(2, 17); printf("%7.1f RPM", encoder2_speed);
		gotoxy(2, 21); printf("%7.1f RPM", encoder3_speed);
		gotoxy(2, 25); printf("%7.1f RPM", encoder4_speed);

		update_vectors();

		gotoxy(1,1); delay(20);	/* just to avoid display flickering */

		if (kbhit()) switch (toupper(getch())) {
			case 'R':											/* reset counters */
				reset_encoders();
				update_vectors();
				break;
			case 'M':
				reset_encoders();
				update_vectors();
				multiplier *= 2;
				if (multiplier > 4) {
					multiplier = 1;											/* multiplier to 1 */
					outportb(base + CONTROL1, QUAD_X1);
					outportb(base + CONTROL3, QUAD_X1);
				}
				if (multiplier == 2) {								/* multiplier to 2 */
					outportb(base + CONTROL1, QUAD_X2);
					outportb(base + CONTROL3, QUAD_X2);
				}
				if (multiplier == 4) {								/* multiplier to 4 */
					outportb(base + CONTROL1, QUAD_X4);
					outportb(base + CONTROL3, QUAD_X4);
				}
				gotoxy(28,3);
				printf("%1d", multiplier);
				break;
			default: 	ok = 0;								/* any other key quits */
		}
	}
	closegraph();
	restorecrtmode();
	return 0;
}


int init(void) {					/* initialize screen and counters */

	int gdriver = VGA, gmode = VGAHI;

	clrscr();
	directvideo = 1;

	printf("US Digital Corp.  PC7266 demo V%s ", version);
	gotoxy(1,3); printf("Enter resolution of encoders (CPR):  ");
	scanf("%ld", &resolution);

	/* disable indexes and IRQs*/
	outportb(base + IRQ1, 0);
	outportb(base + IRQ2, 0);

	if (test()) {
		gotoxy(1,10); printf("Cannot access counters, enter address of PC7266 card (hex): ");
		scanf("%x", &base);
		if (test()) {
			gotoxy(1,12); printf("Cannot access counters, press any key to quit");
			getch();
			return 1;
		}
	}
	/* initialize counters */
	outportb(base + CONTROL1, BP_RESETB);
	outportb(base + CONTROL3, BP_RESETB);
	outportb(base + CONTROL1, INPUT_SETUP);
	outportb(base + CONTROL3, INPUT_SETUP);
	outportb(base + CONTROL1, QUAD_X4);
	outportb(base + CONTROL3, QUAD_X4);

	reset_encoders();

	/* initialize graphics mode */
	initgraph(&gdriver, &gmode, NULL);
	setgraphmode(VGAHI);
	circle(320,240,202);
	/* output text */
	settextjustify(LEFT_TEXT, BOTTOM_TEXT);
	setcolor(LIGHTRED);
	outtextxy(5, 159, "Encoder 1:");
	setcolor(LIGHTBLUE);
	outtextxy(5, 223, "Encoder 2:");
	setcolor(LIGHTGREEN);
	outtextxy(5, 287, "Encoder 3:");
	setcolor(YELLOW);
	outtextxy(5, 351, "Encoder 4:");

	gotoxy(1,1);
	printf("US Digital Corp.  PC7266 demo V%s ", version);
	gotoxy(1,2);
	printf("(Esc) to quit");
	gotoxy(1,3);
	printf("(M) toggle multiplier  M = 4");
	gotoxy(1,4);
	printf("(R)eset counters");

	last_time = clock();
  return 0;
}


void update_vectors(void){	/* update vectors in graphics mode */

	setcolor(BLACK);
	/* erase old lines */
	moveto(320,240);
	linerel((int)(200.0 * sin(encoder1_oldpos * M_PI * 2.0 /(resolution*multiplier))),
					(int)(200.0 * cos(encoder1_oldpos * M_PI * 2.0 /(resolution*multiplier))));
	moveto(320,240);
	linerel((int)(200.0 * sin(encoder2_oldpos * M_PI * 2.0 /(resolution*multiplier))),
					(int)(200.0 * cos(encoder2_oldpos * M_PI * 2.0 /(resolution*multiplier))));
	moveto(320,240);
	linerel((int)(200.0 * sin(encoder3_oldpos * M_PI * 2.0 /(resolution*multiplier))),
					(int)(200.0 * cos(encoder3_oldpos * M_PI * 2.0 /(resolution*multiplier))));
	moveto(320,240);
	linerel((int)(200.0 * sin(encoder4_oldpos * M_PI * 2.0 /(resolution*multiplier))),
					(int)(200.0 * cos(encoder4_oldpos * M_PI * 2.0 /(resolution*multiplier))));
	/* draw new lines */
	setcolor(LIGHTRED);
	moveto(320,240);
	linerel((int)(200.0 * sin(encoder1_pos * M_PI * 2.0 /(resolution*multiplier))),
					(int)(200.0 * cos(encoder1_pos * M_PI * 2.0 /(resolution*multiplier))));
	setcolor(LIGHTBLUE);
	moveto(320,240);
	linerel((int)(200.0 * sin(encoder2_pos * M_PI * 2.0 /(resolution*multiplier))),
					(int)(200.0 * cos(encoder2_pos * M_PI * 2.0 /(resolution*multiplier))));
	setcolor(LIGHTGREEN);
	moveto(320,240);
	linerel((int)(200.0 * sin(encoder3_pos * M_PI * 2.0 /(resolution*multiplier))),
					(int)(200.0 * cos(encoder3_pos * M_PI * 2.0 /(resolution*multiplier))));
	setcolor(YELLOW);
	moveto(320,240);
	linerel((int)(200.0 * sin(encoder4_pos * M_PI * 2.0 /(resolution*multiplier))),
					(int)(200.0 * cos(encoder4_pos * M_PI * 2.0 /(resolution*multiplier))));
}


void read_encoders(void) {		/* position of encoders */

	union pos_tag {	/* allows access of long as 4 bytes */
		long l;
		struct byte_tag {char b0; char b1; char b2; char b3;} byte;
	} pos;

	float minutes;

	/* synchronize with tick clock */
	this_time = clock();
	while (clock() == this_time) {};							/* wait for next tick */
	this_time = clock();

	outportb(base + CONTROL1, TRSFRCNTR_OL);
	outportb(base + CONTROL1, BP_RESETB);
	pos.byte.b0 = inportb(base + DATA1);
	pos.byte.b1 = inportb(base + DATA1);
	pos.byte.b2 = inportb(base + DATA1);
	// extend sign of position
	if (pos.byte.b2 < 0) pos.byte.b3 = -1;
	else pos.byte.b3 = 0;
	encoder1_oldpos =	encoder1_pos;
	encoder1_pos = pos.l;

	outportb(base + CONTROL2, TRSFRCNTR_OL);
	outportb(base + CONTROL2, BP_RESETB);
	pos.byte.b0 = inportb(base + DATA2);
	pos.byte.b1 = inportb(base + DATA2);
	pos.byte.b2 = inportb(base + DATA2);
	// extend sign of position
	if (pos.byte.b2 < 0) pos.byte.b3 = -1;
	else pos.byte.b3 = 0;
	encoder2_oldpos =	encoder2_pos;
	encoder2_pos = pos.l;

	outportb(base + CONTROL3, TRSFRCNTR_OL);
	outportb(base + CONTROL3, BP_RESETB);
	pos.byte.b0 = inportb(base + DATA3);
	pos.byte.b1 = inportb(base + DATA3);
	pos.byte.b2 = inportb(base + DATA3);
	// extend sign of position
	if (pos.byte.b2 < 0) pos.byte.b3 = -1;
	else pos.byte.b3 = 0;
	encoder3_oldpos =	encoder3_pos;
	encoder3_pos = pos.l;

	outportb(base + CONTROL4, TRSFRCNTR_OL);
	outportb(base + CONTROL4, BP_RESETB);
	pos.byte.b0 = inportb(base + DATA4);
	pos.byte.b1 = inportb(base + DATA4);
	pos.byte.b2 = inportb(base + DATA4);
	// extend sign of position
	if (pos.byte.b2 < 0) pos.byte.b3 = -1;
	else pos.byte.b3 = 0;
	encoder4_oldpos =	encoder4_pos;
	encoder4_pos = pos.l;

	if ((this_time - last_time) > 8) {	/* update speed parameters */
		minutes = (float)(this_time - last_time) / (60.0 * CLK_TCK);
		if ((encoder1_pos - encoder1_timepos) > 0X400000)
			encoder1_timepos &= 0X00FFFFFF; /* correct for rollover */
		else if ((encoder1_timepos - encoder1_pos) > 0X400000)
			encoder1_timepos |= 0XFF000000; /* correct for rollover */
		if ((encoder2_pos - encoder2_timepos) > 0X400000)
			encoder2_timepos &= 0X00FFFFFF; /* correct for rollover */
		else if ((encoder2_timepos - encoder2_pos) > 0X400000)
			encoder2_timepos |= 0XFF000000; /* correct for rollover */
		if ((encoder3_pos - encoder3_timepos) > 0X400000)
			encoder3_timepos &= 0X00FFFFFF; /* correct for rollover */
		else if ((encoder3_timepos - encoder3_pos) > 0X400000)
			encoder3_timepos |= 0XFF000000; /* correct for rollover */
		if ((encoder4_pos - encoder4_timepos) > 0X400000)
			encoder4_timepos &= 0X00FFFFFF; /* correct for rollover */
		else if ((encoder4_timepos - encoder4_pos) > 0X400000)
			encoder4_timepos |= 0XFF000000; /* correct for rollover */
		encoder1_speed = (float)(encoder1_pos - encoder1_timepos)
										 / ((float)(resolution*multiplier) * minutes);
		encoder2_speed = (float)(encoder2_pos - encoder2_timepos)
										 / ((float)(resolution*multiplier) * minutes);
		encoder3_speed = (float)(encoder3_pos - encoder3_timepos)
										 / ((float)(resolution*multiplier) * minutes);
		encoder4_speed = (float)(encoder4_pos - encoder4_timepos)
										 / ((float)(resolution*multiplier) * minutes);
		encoder1_timepos = encoder1_pos;
		encoder2_timepos = encoder2_pos;
		encoder3_timepos = encoder3_pos;
		encoder4_timepos = encoder4_pos;
		last_time = this_time;				/* update time */
	}
}


void reset_encoders(void) {	/* reset the counters */

	outportb(base + CONTROL1, CNTR_RESETB);
	outportb(base + CONTROL3, CNTR_RESETB);
	outportb(base + CONTROL1, BP_RESETB);
	outportb(base + CONTROL3, BP_RESETB);
	encoder1_oldpos =	encoder1_pos;	encoder1_pos = 0;
	encoder2_oldpos =	encoder2_pos;	encoder2_pos = 0;
	encoder3_oldpos =	encoder3_pos;	encoder3_pos = 0;
	encoder4_oldpos =	encoder4_pos;	encoder4_pos = 0;
}


int test(void) {	/* test if the counters can be accessed */

	int fail1 = 0, fail2 = 0, fail3 = 0, fail4 = 0;

	gotoxy(1,5);
	/* preset a value and read it back */
	outportb(base + CONTROL1, BP_RESETB);
	outportb(base + DATA1, 0X11);
	outportb(base + DATA1, 0X55);
	outportb(base + DATA1, 0XAA);
	outportb(base + CONTROL1, TRSFRPR_CNTR);
	outportb(base + CONTROL1, TRSFRCNTR_OL);
	outportb(base + CONTROL1, BP_RESETB);
	if (inportb(base + DATA1) != 0X11) fail1++;
	if (inportb(base + DATA1) != 0X55) fail1++;
	if (inportb(base + DATA1) != 0XAA) fail1++;
	if (fail1) printf("\nCounter 1 fails at address %X", base + DATA1);
	/* preset a value and read it back */
	outportb(base + CONTROL2, BP_RESETB);
	outportb(base + DATA2, 0X22);
	outportb(base + DATA2, 0X55);
	outportb(base + DATA2, 0XAA);
	outportb(base + CONTROL2, TRSFRPR_CNTR);
	outportb(base + CONTROL2, TRSFRCNTR_OL);
	outportb(base + CONTROL2, BP_RESETB);
	if (inportb(base + DATA2) != 0X22) fail2++;
	if (inportb(base + DATA2) != 0X55) fail2++;
	if (inportb(base + DATA2) != 0XAA) fail2++;
	if (fail2) printf("\nCounter 2 fails at address %X", base + DATA2);
	/* preset a value and read it back */
	outportb(base + CONTROL3, BP_RESETB);
	outportb(base + DATA3, 0X33);
	outportb(base + DATA3, 0X55);
	outportb(base + DATA3, 0XAA);
	outportb(base + CONTROL3, TRSFRPR_CNTR);
	outportb(base + CONTROL3, TRSFRCNTR_OL);
	outportb(base + CONTROL3, BP_RESETB);
	if (inportb(base + DATA3) != 0X33) fail3++;
	if (inportb(base + DATA3) != 0X55) fail3++;
	if (inportb(base + DATA3) != 0XAA) fail3++;
	if (fail3) printf("\nCounter 3 fails at address %X", base + DATA3);
	/* preset a value and read it back */
	outportb(base + CONTROL4, BP_RESETB);
	outportb(base + DATA4, 0X44);
	outportb(base + DATA4, 0X55);
	outportb(base + DATA4, 0XAA);
	outportb(base + CONTROL4, TRSFRPR_CNTR);
	outportb(base + CONTROL4, TRSFRCNTR_OL);
	outportb(base + CONTROL4, BP_RESETB);
	if (inportb(base + DATA4) != 0X44) fail4++;
	if (inportb(base + DATA4) != 0X55) fail4++;
	if (inportb(base + DATA4) != 0XAA) fail4++;
	if (fail4) printf("\nCounter 4 fails at address %X", base + DATA4);

	return fail1 + fail2 + fail3 + fail4;
}
