
/* LS7266 APPLICATION NOTE */
/* note: pin 18, 19, 28 and 1 are normally high, pin 13 to address 0, pin 17 to address 1*/
/* This program displays the position, flag, which encoder triggered IRQ, and IRQ count */
/* Default is IRQ 5 with only encoder #1 connected(May freeze computer if configured wrong)*/
/* Borland C source code 8/20/96 */

#include <stdio.h>             
#include <dos.h>
#include <conio.h>
#include <process.h>

/* addresses */
#define BASE			0X300				// base address of interface board
#define DATA1			BASE+0			// data register of LS7266 axis X
#define CONTROL1	BASE+1			// control register of LS7266 axis X
#define DATA2			BASE+2			// data register of LS7266 axis Y
#define CONTROL2	BASE+3			// control register of LS7266 axis Y
#define DATA3			BASE+4			// data register of LS7266 axis X
#define CONTROL3	BASE+5			// control register of LS7266 axis X
#define DATA4			BASE+6			// data register of LS7266 axis Y
#define CONTROL4	BASE+7			// control register of LS7266 axis Y
#define IRQ1			BASE+8 			// IRQ control buffer 1
#define IRQ2			BASE+9 			// IRQ control buffer 2
#define IOPORT    BASE+A			// input/output port
#define IRQSTATUS	BASE+8			// read only IRQ status

/* LS7266 commands */
#define CLOCK_DATA 		14			// FCK frequency divider
#define CLOCK_SETUP 	0X98		// transfer PR0 to PSC (x and y)
#define INPUT_SETUP   0XC1		// enable inputs A and B (x and y)
#define QUAD_X1       0XA8		// quadrature multiplier to 1 (x and y)
#define QUAD_X2       0XB0		// quadrature multiplier to 2 (x and y)
#define QUAD_X4       0XB8		// quadrature multiplier to 4 (x and y)
#define BP_RESET    	0X01		// reset byte pointer
#define BP_RESETB  		0X81		// reset byte pointer (x and y)
#define CNTR_RESET    0X02		// reset counter
#define CNTR_RESETB   0X82		// reset counter (x and y)
#define TRSFRPR_CNTR  0X08		// transfer preset register to counter
#define TRSFRCNTR_OL	0X90		// transfer CNTR to OL (x and y)
#define EFLAG_RESET		0X86		// reset E bit of flag register

#ifdef __cplusplus
		#define __CPPARGS ...
#else
		#define __CPPARGS
#endif

/*global variables*/
int count=0, temp=0;
void interrupt far int_handler();
void interrupt (*oldfunc)(__CPPARGS); /* interrupt function pointer */

int init(void) {
	int fail = 0;
	outportb(IRQ1, 0);			// disable index(I1 & I2)
	outportb(IRQ2, 0);			// disable index(I3 & I4)
	inport(0X308);					// reset the outputs of the PAL's

	/*initialize the 7266*/
	outportb(CONTROL1, EFLAG_RESET);		// reset E bit of flag register
	outportb(CONTROL1, BP_RESETB);			// reset byte pointer (x and y)
	outportb(DATA1, CLOCK_DATA);				// FCK frequency divider
	outportb(DATA2, CLOCK_DATA);				// FCK frequency divider
	outportb(CONTROL1, CLOCK_SETUP);		// transfer PR0 to PSC (x and y)
	outportb(CONTROL1, INPUT_SETUP);		// enable inputs A and B (x and y)
	outportb(CONTROL1, QUAD_X4);				// quadrature multiplier to 4 (x and y)
	outportb(CONTROL1, CNTR_RESETB);		// reset counter (x and y)

	outportb(CONTROL3, EFLAG_RESET);		// reset E bit of flag register
	outportb(CONTROL3, BP_RESETB);			// reset byte pointer (x and y)
	outportb(DATA3, CLOCK_DATA);				// FCK frequency divider
	outportb(DATA4, CLOCK_DATA);				// FCK frequency divider
	outportb(CONTROL3, CLOCK_SETUP);		// transfer PR0 to PSC (x and y)
	outportb(CONTROL3, INPUT_SETUP);		// enable inputs A and B (x and y)
	outportb(CONTROL3, QUAD_X4);				// quadrature multiplier to 4 (x and y)
	outportb(CONTROL3, CNTR_RESETB);		// reset counter (x and y)


	/* preset values to the X-axis preset register*/
	outportb(CONTROL1, BP_RESET);				// reset byte pointer
	outportb(DATA1, 0x12);							// output preset value
	outportb(DATA1, 0x34);
	outportb(DATA1, 0x56);
	/*read counter back to test it*/
	outportb(CONTROL1, TRSFRPR_CNTR);		// transfer preset to counter
	outportb(CONTROL1, TRSFRCNTR_OL);		// transfer CNTR to OL
	outportb(CONTROL1, CNTR_RESET);			// reset counter

	outportb(CONTROL1, BP_RESET);				// reset byte pointer
	if (inportb(DATA1) != 0x12) fail++;	// if data is different, 1 is added to fail
	if (inportb(DATA1) != 0x34) fail++;	// if data is different, 1 is added to fail
	if (inportb(DATA1) != 0x56) fail++;
	if (fail) printf("Counter 1 failed\n");	// print message if failed

	/* preset values to the Y-axis preset register*/
	outportb(CONTROL2, BP_RESET);				// reset byte pointer
	outportb(CONTROL2, CNTR_RESET);			// reset counter
	outportb(DATA2, 0x78);							// output preset value
	outportb(DATA2, 0x9A);
	outportb(DATA2, 0xBC);
	/*read counter back to test it*/
	outportb(CONTROL2, TRSFRPR_CNTR);		// transfer preset to counter
	outportb(CONTROL2, TRSFRCNTR_OL);		// transfer CNTR to OL
	outportb(CONTROL2, CNTR_RESETB);		// reset counter (x and y)
	outportb(CONTROL2, BP_RESET);				// reset byte pointer
	if (inportb(DATA2) != 0x78) fail++;	// read output latch
	if (inportb(DATA2) != 0x9A) fail++;	// if data is different, 1 is added to fail
	if (inportb(DATA2) != 0xBC) fail++;
	if (fail) printf("Counter 2 failed\n");	// print message if failed

	/* preset values to the Y-axis preset register*/
	outportb(CONTROL3, BP_RESET);				// reset byte pointer
	outportb(CONTROL3, CNTR_RESET);			// reset counter
	outportb(DATA3, 0x78);							// output preset value
	outportb(DATA3, 0x9A);
	outportb(DATA3, 0xBC);
	/*read counter back to test it*/
	outportb(CONTROL3, TRSFRPR_CNTR);		// transfer preset to counter
	outportb(CONTROL3, TRSFRCNTR_OL);		// transfer CNTR to OL
	outportb(CONTROL3, CNTR_RESETB);		// reset counter (x and y)
	outportb(CONTROL3, BP_RESET);				// reset byte pointer
	if (inportb(DATA3) != 0x78) fail++;	// read output latch
	if (inportb(DATA3) != 0x9A) fail++;	// if data is different, 1 is added to fail
	if (inportb(DATA3) != 0xBC) fail++;
	if (fail) printf("Counter 3 failed\n");	// print message if failed

	/* preset values to the Y-axis preset register*/
	outportb(CONTROL4, BP_RESET);				// reset byte pointer
	outportb(CONTROL4, CNTR_RESET);			// reset counter
	outportb(DATA4, 0x78);							// output preset value
	outportb(DATA4, 0x9A);
	outportb(DATA4, 0xBC);
	/*read counter back to test it*/
	outportb(CONTROL4, TRSFRPR_CNTR);		// transfer preset to counter
	outportb(CONTROL4, TRSFRCNTR_OL);		// transfer CNTR to OL
	outportb(CONTROL4, CNTR_RESETB);		// reset counter (x and y)
	outportb(CONTROL4, BP_RESET);				// reset byte pointer
	if (inportb(DATA4) != 0x78) fail++;	// read output latch
	if (inportb(DATA4) != 0x9A) fail++;	// if data is different, 1 is added to fail
	if (inportb(DATA4) != 0xBC) fail++;
	if (fail) printf("Counter 4 failed\n");	// print message if failed
	return fail;												// return fail variable
}

long read_position(void) {						// read position of encoder
	long position;
	outportb(CONTROL1, BP_RESETB);			// reset byte pointer
	position  = (long)inportb(DATA1);		// least significant byte
	position += (long)inportb(DATA1) << 8;
	position += (long)inportb(DATA1) <<16;	// most significant byte
	return position;
}

long read_position2(void) {						// read position of encoder
	long position;
	outportb(CONTROL2, BP_RESETB);			// reset byte pointer
	position  = (long)inportb(DATA2);		// least significant byte
	position += (long)inportb(DATA2) << 8;
	position += (long)inportb(DATA2) <<16;	// most significant byte
	return position;
}

long read_position3(void) {						// read position of encoder
	long position;
	outportb(CONTROL3, BP_RESETB);			// reset byte pointer
	position  = (long)inportb(DATA3);		// least significant byte
	position += (long)inportb(DATA3) << 8;
	position += (long)inportb(DATA3) <<16;	// most significant byte
	return position;
}

long read_position4(void) {						// read position of encoder
	long position;
	outportb(CONTROL4, BP_RESETB);			// reset byte pointer
	position  = (long)inportb(DATA4);		// least significant byte
	position += (long)inportb(DATA4) << 8;
	position += (long)inportb(DATA4) <<16;	// most significant byte
	return position;
}

int main(void) {
	int mask;

	if (init()) return 0;	 	 					// exit if LS7266 failed

	outportb(IRQ1,0x03); 							// set mode of the interrupt generators
	outportb(IRQ2,0x00); 							// set to "interrupt triggered by index" mode
																		// for encoder #1.  Disabled 2,3,and 4.
	/* save the old interrupt */
	oldfunc  = _dos_getvect(0XD);
	/* install a new interrupt */
	_dos_setvect( 0XD, int_handler); 	// setup new vector
	mask = inportb(0x21);
	mask &= 0xDF;											// turn on IRQ
	outportb(0x21, mask);
	enable();

	printf("\n\n Interrupt installed \n");

	while (!kbhit()) {
		outportb(CONTROL1, TRSFRCNTR_OL);    	// transfer CNTR to OL
		outportb(CONTROL2, TRSFRCNTR_OL);    	// transfer CNTR to OL
		outportb(CONTROL3, TRSFRCNTR_OL);     // transfer CNTR to OL
		outportb(CONTROL4, TRSFRCNTR_OL);     // transfer CNTR to OL
		printf("p1=%7lX ", read_position());  // print position
		printf("p2=%7lX ", read_position2()); // print position
		printf("p3=%7lX ", read_position3()); // print position
		printf("p4=%7lX ", read_position4());	// print position
		printf("f1=%02X ", inportb(CONTROL1)); 	// print position
		printf("f2=%02X ", inportb(CONTROL2)); 	// print position
		printf("f3=%02X ", inportb(CONTROL3)); 	// print position
		printf("f4=%02X ", inportb(CONTROL4)); 	// print position
		printf("IRQ=%X ", count);             	// print IRQ count
		printf("%X\n",temp);
		}

return 0;
}

void interrupt far int_handler()
{
	outportb(0x20, 0X65);		// acknowledge IRQ
	delay(10);            	// delay
	count=inportb(0x308);		// reset status bit of PAL's 1 and 2
	temp++;								 	// add one to variable
}
